/** @file
 *
 * @defgroup looptimus_plus RT9471.c
 * @{
 * @brief    configure and read data from RT9471.
 *
 * This file contains the source code for configuration and .
 *  read data from RT9471.c
 */

#ifndef _RT9471_H__
#define _RT9471_H__

#define BC_IC_I2C_ADD   0x53
#define REG_TOP_ADD     0x01
#define REG_FUN_ADD     0x02
#define REG_IBUS_ADD    0x03
#define REG_VBUS_ADD    0x04
#define REG_PRECHG_ADD  0x05
#define REG_VCHG_ADD    0x07
#define REG_ICHG_ADD    0x08
#define REG_INFO_ADD    0x0B
#define REG_ICSTAT_ADD  0x0F
#define REG_STAT0_ADD   0x10

// TOP Register (0x01)
#define WDT_POS   0x00
#define WDT_MASK  0x03
#define WDT_DISABLE 0x00
#define WDT_TIMER   0x01    // 40S till reset

// Function Register (0x02)
#define BATFET_DIS_DLY_POS  6
#define BATFET_DIS_DLY_MASK 0xC0
#define BATFET_TURN_ON      0x00
#define BATFET_DIS_IMMDTLY  0x02
#define BATFET_DIS_WITH_DLY 0x03

#define CHARGING_EN_POS     0x00
#define CHARGING_EN_MASK    0x01
#define CHARGING_ENABLE     0x01
#define CHARGING_DISABLE    0x00

// VBUS Register (0x04)
#define VAC_OVP_POS         0x06
#define VAC_OVP_MASK        0xC0
#define VAC_OVP_10pt9V      0x02

//#define CHARGE_CUR_350MA    0x07  //NJP ??? 23OCT2023
#define CHARGE_CUR_350MA    0x0F // set 800mA by Niraj Patel

// INFO Register (0x0B)
#define DEVICE_ID_POS       0x03
#define DEVICE_ID_MASK      0x78
#define DEVICE_ID_RT9471    0x0D

// IC STAT Regisetr (0x0F)
// PORT_STAT
// IC_STAT
#define PORT_STAT_MASK      0x0F
#define PORT_STAT_POS       0x04
#define PORT_STAT_NOINFO    0x00
#define PORT_STAT_PSEL_HIGH 0x0D

#define IC_STAT_MASK        0x0F
#define IC_STAT_READY4CHRG  0x01
#define IC_STAT_TRICKLE     0x02
#define IC_STAT_PRECHARGE   0x03
#define IC_STAT_FASTCHARGE  0x04
#define IC_STAT_IEOCCHARGE  0x05
#define IC_STAT_BKGCHARGE   0x06
#define IC_STAT_CHARGE_DONE 0x07
#define IC_STAT_CHARGE_FLT  0x08

// STAT0 Register (0x10)
// VBUS Good
// VBUS Ready
#define ST_VBUS_GD_POS      0x07
#define ST_VBUS_GD_MASK     0x01
#define ST_CHG_RDY_POS      0x06
#define ST_CHG_RDY_MASK     0x01

#define LED_DEVICE_ON_EXTPOWER  BAT_GREEN_LED_NUM
#define LED_DEVICE_ON_BATTERY   BAT_RED_LED_NUM

#define TURN_ON_EXTERNAL_POWER_INDICATION_LED  bsp_board_led_on(LED_DEVICE_ON_EXTPOWER);
#define TURN_OFF_EXTERNAL_POWER_INDICATION_LED bsp_board_led_off(LED_DEVICE_ON_EXTPOWER);
#define TURN_ON_BATTERY_POWER_INDICATION_LED   bsp_board_led_on(LED_DEVICE_ON_BATTERY);
#define TURN_OFF_BATTERY_POWER_INDICATION_LED  bsp_board_led_off(LED_DEVICE_ON_BATTERY);
#define TOGGLE_EXTERNAL_POWER_INDICATION_LED   (bsp_board_led_state_get(LED_DEVICE_ON_EXTPOWER) ? \
                                                bsp_board_led_off(LED_DEVICE_ON_EXTPOWER) : \
                                                bsp_board_led_on(LED_DEVICE_ON_EXTPOWER));


//global veriable


void RT9471_init(void);
uint8_t Verify_RT9471 (void);
uint8_t BC_EnableDisable_Charging (uint8_t enable);
uint8_t BC_Update_BATFET (uint8_t value);
uint8_t BC_EnableDisable_Watchdog (uint8_t value);
uint8_t BC_Update_VOP (uint8_t value);
void BC_StartBATFET (void);

#endif    //#ifndef _RT9471_H__