/** @file
 *
 * @defgroup looptimus_plus looptimus_app.c
 * @{
 * @brief    main functionality of Looptimus Plus device.
 *
 * This file contains the source code for application that looptimus plus
 * device requires to do.
 * 
 */

#ifndef LOOPTIMUS_PLUS_CONFIG_H__
#define LOOPTIMUS_PLUS_CONFIG_H__

#include "looptimus_mini_plus_app.h"

#define SW_VER  "001"

#define EXP_PEDAL_CAL_LOW_POS     0
#define EXP_PEDAL_CAL_HIGH_POS    2

#define DEF_CHNL_VAL_POS          0
#define PCC_CHNL_VAL_POS          1
#define PAD_CHNL_VAL_POS          2
#define MODE_VAL_POS              3

#define OCTAVE_VAL_POS            3
#define BRIGHTNETSS_VAL_POS       2

#define PCMAP_1ST_VAL_POS         0
#define PCMAP_2ND_VAL_POS         1
#define PCMAP_3RD_VAL_POS         2
#define PCMAP_4TH_VAL_POS         3

#define EXT_MODE_1ST_CHNL_POS     0
#define EXT_MODE_2ND_CHNL_POS     1
#define EXT_MODE_3RD_CHNL_POS     2
#define EXTMODE_VAL_POS           3

#ifdef CC_MODE_SUPPORT
#define CCMAP_1ST_VAL_POS         0
#define CCMAP_2ND_VAL_POS         1
#define CCMAP_3RD_VAL_POS         2
#define CCMAP_4TH_VAL_POS         3

#define EXT_MODE_CC_1ST_CHNL_POS  0
#define EXT_MODE_CC_2ND_CHNL_POS  1
#define EXT_MODE_CC_3RD_CHNL_POS  2
#define CC_MODE_CHNL_VAL_POS      3
#endif      //#ifdef CC_MODE_SUPPORT

// With ADC Config: Ref: Internal, Gain: 1/6, Bit: 10
#define EXT_PEDAL_3V3_ADC_VAL    938   // 3.3 V
#define EXT_PEDAL_0V_ADC_VAL      0    // 0V

#define NO_SUB_PARAM    0

typedef enum {
    PARAM_EXP_PAD_CAL_LOW = 0,    // Expression Pedal calibration low value
    PARAM_EXP_PAD_CAL_HIGH = 1,   // Expression Pedal calibration high value
    PARAM_MODE = 2,               // Mode value - Default, PCC or PAD
    PARAM_DEFAULT_MODE = 3,        // Defualt mode set
    PARAM_EXT_PAD_MODE = 4,       // Extension pedal mode value
    PARAM_EXT_PAD_PCC = 5,        // Extension pedal channel value
    PARAM_KEYBOARD_BOND = 6,
    PARAM_KEYBOARD_MODE = 7,
#ifdef CC_MODE_SUPPORT
    PARAM_CC_MODE_CHNL = 11,      // Channel value in CC mode
    PARAM_CC_MAP = 12,            // PCC Mapping values
    PARAM_EXT_PAD_CC = 13        // Extension pedal channel value
#endif
} configParam_t;

typedef struct //looptimus_config_data
{
   uint32_t mExpPedalCalibration;     // Expression pedal calibration: HIGH(2) + LOW(2)
   uint32_t mMode_midi;               // MiDi Mode - 1 byte each
   uint32_t keyboard_bond_state;
   uint32_t mMode_keyboard;
#ifdef CC_MODE_SUPPORT
   uint32_t mCCMap[32];               // PCC mapping - 126 bytes required - 2 reserved
   uint32_t mExtModeCCMapChnl;         // Extension pedal mode - 1 byte, PCC mapping for extension pedal - 3 bytes
#endif      //#ifdef CC_MODE_SUPPORT
} looptimus_config_data_t;

extern looptimus_config_data_t gLooptimusConfigData;

void CheckNCorrectChannels (void);
//void CheckNCorrectNotesForPCMode (byte temp);
#ifdef CC_MODE_SUPPORT
void CheckNCorrectNotesForCCMode (byte temp);
#endif
void GetParam_FromMemory (byte paramNumber, byte arg, void *value);
void Init_DefaultConfigData (void);
void UpdateAllConfigData (looptimus_config_data_t *pConfigData);
void UpdateConfigData (byte paramNumber, byte arg, void *value);
byte Mem_ReadCurChannel (byte mode);
void ReadNUpdateConfigToDefault (void);
void Init_Configuration(void);

#endif  //LOOPTIMUS_PLUS_CONFIG_H__