/** @file
 *
 * @defgroup measure_adc.c
 * @{
 * @brief    Function for measuring Analog values from Analog channel.
 *
 * This file contains the source code for measuring adc value from switch.
 * 
 */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "definition.h"

static nrf_saadc_value_t     m_buffer_pool[SAMPLES_IN_BUFFER];
volatile bool                vIsAdcConvDone;
typedef uint8_t byte;

boolean gsInitDone = false;

byte batvoltpin = BAT_VOLT_PIN; //Analog pin # for read battery voltage

/**@brief Function to average data of three ADC samples.
 *
 * @param[in]   p_buffer    Buffer having ADC sample data.
 * @param[out]  val         average of three ADC sample data.
 */
int AverageADC (nrf_saadc_value_t *p_buffer)
{
    ret_code_t err_code;
    int val = 0;

    val = (p_buffer[0] + p_buffer[1] + p_buffer[2]);
    val = (val / 3);

    return val;
}


/**@brief callback function when ADC sampling is done.
 *
 * @param[in]   p_event    variable having details of ADC events.
 */

static void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
    {
        ret_code_t err_code;
        BaseType_t xHigherPriorityTaskWoken = pdFALSE;

        /* Release the semaphore */
        vIsAdcConvDone = true;
        
        if (gsInitDone)
        {
            vTaskNotifyGiveFromISR(looptimus_mini_plus_app_thread, &xHigherPriorityTaskWoken);
            portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
        }
        
    }

    nrfx_saadc_uninit ();
    NRF_SAADC->INTENCLR = (SAADC_INTENCLR_END_Clear << SAADC_INTENCLR_END_Pos);
    NVIC_ClearPendingIRQ(SAADC_IRQn);

}


/**@brief Function to init ADC and start sampling on given channel.
 *
 * @param[in]   channel    ADC channel to take samples from.
 */
void get_adc_sample(int channel)
{
   ret_code_t err_code;
   // config analog channel (i.e from AIN0 - AIN7)
   nrf_saadc_channel_config_t channel_config = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(channel);
   vIsAdcConvDone = false;
#ifdef ADC_USE_CALLBACK
    err_code = nrf_drv_saadc_init(NULL, saadc_callback);
#else
    err_code = nrf_drv_saadc_init(NULL, NULL);
#endif
    APP_ERROR_CHECK(err_code);  
    err_code = nrf_drv_saadc_channel_init(0, &channel_config);
    APP_ERROR_CHECK(err_code);
    nrf_saadc_continuous_mode_enable(80);
    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool, SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);
    nrfx_saadc_sample ();
    byte i;
    for (i = 0; i < 1; i++)
    {
        //nrfx_saadc_sample ();
        for (uint16_t j = 0; j < 50; j++)
            __NOP();
    }
}

int measure_battery_volt()
{
    int read_value = 0;
    get_adc_sample(batvoltpin);
    while (vIsAdcConvDone == false);
    read_value = AverageADC(m_buffer_pool);
    return read_value;
}

void Process_midi_Buttons(void)
{
        int gadc_value = 0;
        get_adc_sample(SAADC_BTN_PIN);
        while (vIsAdcConvDone == false);
        gadc_value = AverageADC(m_buffer_pool);
        Check_button_status(gadc_value);
}

/* Read button status at Power On Condition */
int read_button_at_PoS (void)
{
        int gread_adc = 0;
        get_adc_sample(SAADC_BTN_PIN);
        while (vIsAdcConvDone == false);
        gread_adc = AverageADC(m_buffer_pool);
        return gread_adc;
}

/* Read external pedal analog values */
int read_ext_pedal_value(void)
{
        int ext_pedal_val = 0;
        get_adc_sample(EXT_PEDAL_PIN);
        while (vIsAdcConvDone == false);
        ext_pedal_val = AverageADC(m_buffer_pool);
        return ext_pedal_val;
}