#include "nrf_gpio.h"
#include "custom_board.h"
#include "cmsis_gcc.h"

#include "user_drv_i2c.h"
//#include "nrf_delay.h"

   
#if 0
#define I2C_ANSEL      ANSELF
#define I2C_INPORT     PORTF
#define I2C_TRIS       TRISF

#define I2C_SET_SCL    I2C_TRIS |= SCL;
#define I2C_CLR_SCL    I2C_TRIS &= ~SCL;
#define I2C_SET_SDA    I2C_TRIS |= SDA;
#define I2C_CLR_SDA    I2C_TRIS &= ~SDA;
#endif

#define I2C_SET_SCL    nrf_gpio_pin_set(DISPLAY_SCL_PIN);
#define I2C_CLR_SCL    nrf_gpio_pin_clear(DISPLAY_SCL_PIN);
#define I2C_SET_SDA    nrf_gpio_pin_set(DISPLAY_SDA_PIN);
#define I2C_CLR_SDA    nrf_gpio_pin_clear(DISPLAY_SDA_PIN);

#define I2C_INPORT     nrf_gpio_pin_read(DISPLAY_SDA_PIN)

void user_delay(void)
{
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
}


void i2c_init(void)
{
    I2C_SET_SCL
    I2C_SET_SDA

    I2C_DELAY
}


//Initiating an I2C start condition:
void i2c_start_condition( void )
{
    I2C_SET_SCL
    I2C_SET_SDA
    I2C_DELAY
    I2C_CLR_SDA
    I2C_DELAY
    I2C_CLR_SCL
    I2C_DELAY
}


// Initiating an I2C stop condition:
void i2c_stop_condition( void )
{
    I2C_CLR_SDA
    I2C_DELAY
    I2C_SET_SCL
    I2C_DELAY
    I2C_SET_SDA
    I2C_DELAY
}


// Writing a bit in I2C:
void i2c_write_bit( uint8_t b )
{
    if( b > 0 )
        I2C_SET_SDA
    else
        I2C_CLR_SDA

    I2C_DELAY
    I2C_SET_SCL
    I2C_DELAY
    I2C_CLR_SCL
}


// Reading a bit in I2C:
uint8_t i2c_read_bit( void )
{
    uint8_t b;
     
    I2C_SET_SDA
    I2C_DELAY
    I2C_SET_SCL
    I2C_DELAY

    if( I2C_INPORT )
    {
        b = 0;
    }
    else
    {
        b = 1;
    }
    
    I2C_CLR_SCL

    I2C_SET_SDA
    I2C_CLR_SDA
    I2C_CLR_SDA

    return b;
}


// Writing a byte with I2C:
bool i2c_write_byte( uint8_t B,
                     bool start,
                     bool stop )
{
    uint8_t ack = 0;

    if( start )
        i2c_start_condition();

    uint8_t i;
    for( i = 0; i < 8; i++ )
    {
        i2c_write_bit( B & 0x80 );   // write the most-significant bit
        B <<= 1;
    }
    
    ack = i2c_read_bit();

    if( stop )
        i2c_stop_condition();
    
    return ack;
}


// Reading a byte with I2C:
uint8_t i2c_read_byte( bool ack,
                       bool stop )
{
    uint8_t B = 0;

    uint8_t i;
    
    for( i = 0; i < 8; i++ )
    {
        B <<= 1;
        B |= i2c_read_bit();
//        I2C_DELAY
        
    }

//  Master Acknowledment
    if( ack )
        i2c_write_bit(0);
    else
        i2c_write_bit(1);
////////////////////////
    if( stop )
        i2c_stop_condition();

    return B;
}


// Sending a byte with I2C:
bool i2c_send_byte( uint8_t address,
                    uint8_t data )
{
    if( i2c_write_byte( address << 1, true, false ) )   // start, send address, write
    {
        // send data, stop
        if( i2c_write_byte( data, false, true ) )
            return true;
    }
    
    i2c_stop_condition();   // make sure to impose a stop if NAK'd
    return false;
}


// Receiving a byte with a I2C:
uint8_t i2c_receive_byte( uint8_t address )
{
    if( i2c_write_byte( ( address << 1 ) | 0x01, true, false ) )   // start, send address, read
    {
        return i2c_read_byte( false, true );
    }

    return 0;   // return zero if NAK'd
}

// Sending a byte of data with I2C:
bool i2c_send_byte_data( uint8_t address,
                         uint8_t reg,
                         uint8_t data )
{
    if( i2c_write_byte( address << 1, true, false ) )   // start, send address, write
    {
        if( i2c_write_byte( reg, false, false ) )   // send desired register
        {
            if( i2c_write_byte( data, false, true ) ) return true;   // send data, stop
        }
    }

    i2c_stop_condition();
    return false;
}

// Receiving a byte of data with I2C:
uint8_t i2c_receive_byte_data( uint8_t address,
                               uint8_t reg )
{
    if( i2c_write_byte( address << 1, true, false ) )   // start, send address, write
    {
        if( i2c_write_byte( reg, false, false ) )   // send desired register
        {
            if( i2c_write_byte( ( address << 1) | 0x01, true, false ) )   // start again, send address, read
            {
                return i2c_read_byte( false, true );   // read data
            }
        }
    }

    i2c_stop_condition();
    return 0;   // return zero if NACKed
}